use ark_bls12_381::Fr;
use ark_ff::PrimeField;
use ark_poly::univariate::DensePolynomial;
use ark_poly::DenseUVPolynomial;
use ark_std::ops::Div;
use rs_merkle::Hasher;

use fri::algorithms::Blake3;
use komodo::error::KomodoError;

fn run<const N: usize, F: PrimeField, H: Hasher, P>(
    bytes: &[u8],
    k: usize,
    n: usize,
    bf: usize,
    rpo: usize,
    q: usize,
) -> Result<(), KomodoError>
where
    P: DenseUVPolynomial<F>,
    for<'a, 'b> &'a P: Div<&'b P, Output = P>,
    <H as rs_merkle::Hasher>::Hash: AsRef<[u8]>,
{
    let shards = komodo::fri::encode::<F>(&bytes, k, n);
    let blocks = komodo::fri::prove::<N, F, H, P>(shards, bf, rpo, q).unwrap();

    for block in blocks {
        komodo::fri::verify::<N, F, H, P>(block, n, q).unwrap();
    }

    Ok(())
}

fn main() {
    let bytes = include_bytes!("../assets/dragoon_133x133.png").to_vec();
    eprintln!("loaded {} bytes of data", bytes.len());

    run::<2, Fr, Blake3, DensePolynomial<Fr>>(&bytes, 4, 16, 1, 1, 50).unwrap();
}
